﻿using Android.App;
using Android.Content;
using Android.Graphics;
using Android.OS;
using Android.Runtime;
using Android.Support.V7.App;
using Android.Views;
using Android.Widget;
using Syncfusion.SfDataGrid;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace SFGridTest
{
    [Activity(Label = "@string/app_name", Theme = "@style/AppTheme", MainLauncher = true)]
    public class MainActivity : AppCompatActivity
    {

        private LinearLayout _contentLayout;
        private int _gridCount = 1;
        

        protected override void OnCreate(Bundle savedInstanceState)
        {
            base.OnCreate(savedInstanceState);
            Xamarin.Essentials.Platform.Init(this, savedInstanceState);

            // Create ScrollView with ContentLayout
            var rootScrollView = new ScrollView(this);
            rootScrollView.LayoutParameters = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MatchParent, ViewGroup.LayoutParams.MatchParent);
            rootScrollView.SetBackgroundColor(Color.BlueViolet);
            _contentLayout = new LinearLayout(this);
            _contentLayout.LayoutParameters = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MatchParent, ViewGroup.LayoutParams.WrapContent);
            _contentLayout.Orientation = Orientation.Vertical;
            rootScrollView.AddView(_contentLayout);

            SetContentView(rootScrollView);
        }



        protected override void OnStart()
        {
            base.OnStart();
            SetContent();
        }

        // Create Grids
        private void SetContent()
        {
            int i = 0;
            while (i < _gridCount)
            {
                CreateDataGrid();
                i++;
            }
        }


        /// <summary>
        /// Create a GridView
        /// </summary>
        /// <param name="gridHeight"></param>
        private void CreateDataGrid(int gridHeight = 400)
        {
            var sfDataGrid = new SfDataGrid(this);
            var gridParams = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MatchParent, gridHeight);
            gridParams.SetMargins(20, 20, 20, 20);
            sfDataGrid.LayoutParameters = gridParams;
            sfDataGrid.AutoGenerateColumns = false;
            sfDataGrid.ColumnSizer = ColumnSizer.Star;
            sfDataGrid.AllowPullToRefresh = false;
            sfDataGrid.AllowResizingColumn = true;
            sfDataGrid.AllowDraggingColumn = false;
            sfDataGrid.AllowSorting = true;
            sfDataGrid.AllowTriStateSorting = true;
            sfDataGrid.GridStyle = new GrayWhiteStyle();
            sfDataGrid.SelectionMode = SelectionMode.Single;
            sfDataGrid.AllowDraggingRow = false;
            sfDataGrid.AllowMultiSorting = true;

            CreateColumns(sfDataGrid);

            var _sfDataVM = new SfDataViewModel();
            sfDataGrid.ItemsSource = _sfDataVM.Collection;
            //sfDataGrid.QueryRowHeight += DataGrid_QueryRowHeight;

            _contentLayout.AddView(sfDataGrid);
        }

        /*
        private void DataGrid_QueryRowHeight(object sender, QueryRowHeightEventArgs e)
        {
            if (e.RowIndex != 0)
            {
                e.Height = (sender as SfDataGrid).GetRowHeight(e.RowIndex);
                e.Handled = true;
            }
        }
        */

        /// <summary>
        /// Creates the Grid Column with Mapping Names
        /// </summary>
        /// <param name="grid"></param>
        private void CreateColumns(SfDataGrid grid)
        {
            // Works fine
            /*
            grid.Columns.Add(new GridTextColumn() { HeaderText = "Field01", MappingName = "Field01", AllowSorting = true, AllowEditing = false });
            grid.Columns.Add(new GridTextColumn() { HeaderText = "Field02", MappingName = "Field02", AllowSorting = true, AllowEditing = false });
            grid.Columns.Add(new GridTextColumn() { HeaderText = "Field03", MappingName = "Field03", AllowSorting = true, AllowEditing = false });
            grid.Columns.Add(new GridTextColumn() { HeaderText = "Field04", MappingName = "Field04", AllowSorting = true, AllowEditing = false });
            grid.Columns.Add(new GridTextColumn() { HeaderText = "Field05", MappingName = "Field05", AllowSorting = true, AllowEditing = false });
            grid.Columns.Add(new GridTextColumn() { HeaderText = "Field06", MappingName = "Field06", AllowSorting = true, AllowEditing = false });
            grid.Columns.Add(new GridTextColumn() { HeaderText = "Field07", MappingName = "Field07", AllowSorting = true, AllowEditing = false });
            */

            // Custom Columns
            List<string> columns = new List<string>() { "Field01", "Field02" , "Field03" , "Field04" , "Field05" , "Field06" , "Field07" };

            foreach(var col in columns)
            {
                GridTextColumn customColumn = new GridTextColumn();
                customColumn.UserCellType = typeof(CustomCell);
                customColumn.MappingName = col;
                grid.Columns.Add(customColumn);


                // Create a custom column template
                var columnView = new LinearLayout(this);
                columnView.LayoutParameters = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MatchParent, ViewGroup.LayoutParams.MatchParent);
                columnView.Orientation = Orientation.Horizontal;
                columnView.SetGravity(GravityFlags.CenterVertical);

                ImageView columnIcon = new ImageView(this);
                columnIcon.LayoutParameters = new LinearLayout.LayoutParams(50, 50);
                columnIcon.SetImageResource(Resource.Mipmap.ic_whiteSquare);
                columnIcon.SetColorFilter(Color.Green);
                columnView.AddView(columnIcon);
                TextView columnText = new TextView(this);
                columnText.LayoutParameters = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MatchParent, ViewGroup.LayoutParams.WrapContent) { Weight = 1 };
                columnText.SetTextColor(Color.Black);
                columnText.Text = col;
                columnView.AddView(columnText);

                customColumn.HeaderTemplate = columnView;
            }
        }




        public override void OnRequestPermissionsResult(int requestCode, string[] permissions, [GeneratedEnum] Android.Content.PM.Permission[] grantResults)
        {
            Xamarin.Essentials.Platform.OnRequestPermissionsResult(requestCode, permissions, grantResults);

            base.OnRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }

    public class CustomCell : GridCell
    {
        TextView textView;

        public CustomCell(Context context) : base(context)
        {
            textView = new TextView(this.Context);
            textView.SetTextColor(Color.Black);
            AddView(textView);
            CanRendererUnload = false;
        }

        protected override void UnLoad()
        {
            if (this.Parent != null)
                (this.Parent as VirtualizingCellsControl).RemoveView(this);
        }

        protected override void OnLayout(bool change, int l, int t, int r, int b)
        {
            textView.Layout(0, 0, this.Width, this.Height);
        }
        protected override void OnDraw(Canvas canvas)
        {
            base.OnDraw(canvas);
            this.textView.Text = DataColumn.CellValue.ToString();
        }

    }


    /// <summary>
    /// DummyClass for GridItems
    /// </summary>
    public class DummyGridItem
    {
        public string Field01 { get; set; }
        public string Field02 { get; set; }
        public string Field03 { get; set; }
        public string Field04 { get; set; }
        public string Field05 { get; set; }
        public string Field06 { get; set; }
        public string Field07 { get; set; }

        public DummyGridItem(string f1, string f2, string f3, string f4, string f5, string f6, string f7)
        {
            Field01 = f1;
            Field02 = f2;
            Field03 = f3;
            Field04 = f4;
            Field05 = f5;
            Field06 = f6;
            Field07 = f7;
        }
    }

    /// <summary>
    /// Viewmodel zur Erstellung der ItemSource
    /// </summary>
    public class SfDataViewModel
    {
        public SfDataViewModel()
        {
            GenerateRows();
        }

        private ObservableCollection<DummyGridItem> collection;

        public ObservableCollection<DummyGridItem> Collection
        {
            get { return collection; }
            set { this.collection = value; }
        }

        /// <summary>
        /// Erstellt die Collection initial
        /// </summary>
        public void GenerateRows()
        {
            collection = new ObservableCollection<DummyGridItem>();
            int i = 1;
            while (i < 1000)
            {
                collection.Add(new DummyGridItem($"{i}-Field01-long-text-test-for-row-height", $"{i}-Field02", $"{i}-Field03", $"{i}-Field04", $"{i}-Field05", $"{i}-Field06", $"{i}-Field07"));
                i++;
            }
        }
       
    }


    /// <summary>
    /// Alternating Style
    /// </summary>
    public class GrayWhiteStyle : DataGridStyle
    {
        public GrayWhiteStyle() { }

        public override Color GetAlternatingRowBackgroundColor()
        {
            return Color.LightGray;
        }
    }
}